@@ -7,16 +7,17 @@
 #' @param query Any query arguments, passed as a named list of key-value pairs.
 #' @param headers A list of request headers for the REST call.   
 #' @param request_body A character string containing request body data.
-#' @param write_disk If \code{verb = "GET"}, this is, Ootionally, an argument like \code{\link[httr]{write_disk}} to write the result directly to disk.
+#' @param write_disk If \code{verb = "GET"}, this is, optionally, an argument like \code{\link[httr]{write_disk}} to write the result directly to disk.
+#' @param write_fn If set to a function and \code{verb = "GET"} is used then the output is passed in chunks as a raw vector in the first argument to this function, allowing streaming output. Note that \code{write_disk} and \code{write_fn} are mutually exclusive.
 #' @param accelerate A logical indicating whether to use AWS transfer acceleration, which can produce significant speed improvements for cross-country transfers. Acceleration only works with buckets that do not have dots in bucket name.
 #' @param dualstack A logical indicating whether to use \dQuote{dual stack} requests, which can resolve to either IPv4 or IPv6. See \url{http://docs.aws.amazon.com/AmazonS3/latest/dev/dual-stack-endpoints.html}.
 #' @param parse_response A logical indicating whether to return the response as is, or parse and return as a list. Default is \code{TRUE}.
 #' @param check_region A logical indicating whether to check the value of \code{region} against the apparent bucket region. This is useful for avoiding (often confusing) out-of-region errors. Default is \code{FALSE}.
 #' @param url_style A character string specifying either \dQuote{path} (the default), or \dQuote{virtual}-style S3 URLs.
-#' @param base_url A character string specifying the base URL for the request. There is no need to set this, as it is provided only to generalize the package to (potentially) support S3-compatible storage on non-AWS servers. The easiest way to use S3-compatible storage is to set the \env{AWS_S3_ENDPOINT} environment variable.
+#' @param base_url A character string specifying the base hostname for the request (it is a misnomer, the actual URL is constructed from this name, region and \code{use_https} flag. There is no need to set this, as it is provided only to generalize the package to (potentially) support S3-compatible storage on non-AWS servers. The easiest way to use S3-compatible storage is to set the \env{AWS_S3_ENDPOINT} environment variable. When using non-AWS servers, you may also want to set \code{region=""}.
 #' @param verbose A logical indicating whether to be verbose. Default is given by \code{options("verbose")}.
 #' @param show_progress A logical indicating whether to show a progress bar for downloads and uploads. Default is given by \code{options("verbose")}.
-#' @param region A character string containing the AWS region. Ignored if region can be inferred from \code{bucket}. If missing, defaults to \dQuote{us-east-1}.
+#' @param region A character string containing the AWS region. Ignored if region can be inferred from \code{bucket}. If missing, an attempt is made to locate it from credentials. Defaults to \dQuote{us-east-1} if all else fails. Should be set to \code{""} when using non-AWS endpoints that don't include regions (and \code{base_url} must be set).
 #' @param key A character string containing an AWS Access Key ID. If missing, defaults to value stored in environment variable \env{AWS_ACCESS_KEY_ID}.
 #' @param secret A character string containing an AWS Secret Access Key. If missing, defaults to value stored in environment variable \env{AWS_SECRET_ACCESS_KEY}.
 #' @param session_token Optionally, a character string containing an AWS temporary Session Token. If missing, defaults to value stored in environment variable \env{AWS_SESSION_TOKEN}.
@@ -37,6 +38,7 @@ function(verb = "GET",
          headers = list(), 
          request_body = "",
          write_disk = NULL,
+         write_fn = NULL,
          accelerate = FALSE,
          dualstack = FALSE,
          parse_response = TRUE, 
@@ -57,7 +59,8 @@ function(verb = "GET",
     key <- credentials[["key"]]
     secret <- credentials[["secret"]]
     session_token <- credentials[["session_token"]]
-    region <- credentials[["region"]]
+    ## allow region="" to override any config - the only way to use 3rd party URLs without region
+    region <- if (length(region) && !nzchar(region)) region else credentials[["region"]]
     
     # handle 'show_progress' argument
     if (isTRUE(show_progress)) {
@@ -74,14 +77,14 @@ function(verb = "GET",
     bucketname <- get_bucketname(bucket)
     if (isTRUE(check_region) && (bucketname != "")) {
         if (isTRUE(verbose)) {
-            message(sprintf("Checking bucket region using get_location('%s')", bucketname))
+            message("Checking bucket region using get_location('", bucketname, "')")
         }
         bucketregion <- get_region(x = bucket, key = key, secret = secret, session_token = session_token, ...)
         if (!is.null(bucketregion) && bucketregion != "") {
             region <- bucketregion
         }
         if (isTRUE(verbose)) {
-            message(sprintf("Executing request using bucket region %s", region))
+            message("Executing request using bucket region ", region)
         }
     }
     
@@ -144,10 +147,13 @@ function(verb = "GET",
     # execute request
     if (verb == "GET") {
         # GET verb
-        if (!is.null(write_disk)) {
-            r <- httr::GET(url, H, query = query, write_disk, show_progress, ...)
+        r <- if (is.function(write_fn)) {
+            if (!is.null(write_disk)) stop("write_stream and write_disk are mutually exclusive.")
+            httr::GET(url, H, query = query, httr::write_stream(write_fn), show_progress, ...)
+        } else if (!is.null(write_disk)) {
+            httr::GET(url, H, query = query, write_disk, show_progress, ...)
         } else {
-            r <- httr::GET(url, H, query = query, show_progress, ...)
+            httr::GET(url, H, query = query, show_progress, ...)
         }
     } else if (verb == "connection") {
         # support for a streaming GET connection
@@ -363,7 +369,7 @@ function(bucketname,
         url <- paste0(url, "/")
     }
     if (isTRUE(verbose)) {
-        message(sprintf("S3 Request URL: %s", url))
+        message("S3 Request URL: ", url)
     }
     return(url)
 }
